﻿/* 
    Copyright (c) Microsoft Corporation. All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all code Samples for Windows Store apps and Windows Phone Store apps, visit http://code.msdn.microsoft.com/windowsapps
  
*/

namespace Bookstore3WPSL8
{
	using Microsoft.Phone.Globalization;
	using SdkHelper;
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Globalization;
	using System.Linq;
	using System.Windows.Media;
	using System.Windows.Media.Imaging;

	public class GroupByHelper<T> : List<T>
	{
		/// <summary>
		/// The delegate that is used to get the key information.
		/// </summary>
		/// <param name="item">An object of type T</param>
		/// <returns>The key value to use for this object</returns>
		public delegate string GetKeyDelegate(T item);

		/// <summary>
		/// The Key of this group.
		/// </summary>
		public string Key { get; private set; }

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="key">The key for this group.</param>
		public GroupByHelper(string key)
		{
			this.Key = key;
		}

		public GroupByHelper(IGrouping<string, T> grouping)
			: base(grouping)
		{
			this.Key = grouping.Key;
		}

		/// <summary>
		/// Create a list of GroupByHelper<T> grouped by keys set by a SortedLocaleGrouping.
		/// </summary>
		/// <param name="items">The items to place in the groups.</param>
		/// <param name="getKey">A delegate to get the key from an item.</param>
		/// <param name="sort">Will sort the data if true.</param>
		/// <returns>An items source for a LongListSelector</returns>
		public static List<GroupByHelper<T>> CreateSortedLocaleGroupingByInitial(IEnumerable<T> listOfItems, CultureInfo ci, GetKeyDelegate getKey, bool sort)
		{
			// Create a list of GroupByHelper<T> with keys set by a SortedLocaleGrouping.
			SortedLocaleGrouping sortedLocaleGrouping = new SortedLocaleGrouping(ci);
			List<GroupByHelper<T>> listOfGroups = new List<GroupByHelper<T>>();
			foreach (string key in sortedLocaleGrouping.GroupDisplayNames)
			{
				listOfGroups.Add(new GroupByHelper<T>(key));
			}

			foreach (T item in listOfItems)
			{
				int index = 0;
				index = sortedLocaleGrouping.GetGroupIndex(getKey(item));
				if (index >= 0 && index < listOfGroups.Count)
				{
					listOfGroups[index].Add(item);
				}
			}

			if (sort)
			{
				foreach (GroupByHelper<T> group in listOfGroups)
				{
					group.Sort((c0, c1) => { return ci.CompareInfo.Compare(getKey(c0), getKey(c1)); });
				}
			}

			return listOfGroups;
		}

		/// <summary>
		/// Create a list of GroupByHelper<T> grouped by the keys present in the data set provided.
		/// </summary>
		/// <param name="items">The items to place in the groups.</param>
		/// <param name="getKey">A delegate to get the key from an item.</param>
		/// <returns>An items source for a LongListSelector</returns>
		public static List<GroupByHelper<T>> CreateSortedGroupingByKeysPresent(IEnumerable<T> listOfItems, GetKeyDelegate getKey)
		{
			var listOfGroupsToReturn =
							from item in listOfItems
							group item by getKey(item) into grouping
							orderby grouping.Key
							select new GroupByHelper<T>(grouping);

			return new List<GroupByHelper<T>>(listOfGroupsToReturn);
		}
	}

	public class BookstoreViewModel : PropertyChangedNotifier
	{
		#region fields
		private RelayCommand atticusCommand;
		private List<GroupByHelper<BookSku>> authors;
		private ObservableCollection<BookSku> bookSkus;
		private List<GroupByHelper<BookSku>> genres;
		private bool hasAtticusBeenUpdated = false;
		private bool isMobyDickInTheStore = false;
		private RelayCommand mobyDickCommand;
		#endregion fields

		#region properties
		public string AppName
		{
			get
			{
				return "BOOKSTORE3WPSL8";
			}
		}

		public List<GroupByHelper<BookSku>> Authors
		{
			get
			{
				if (this.authors == null)
				{
					this.authors = GroupByHelper<BookSku>.CreateSortedLocaleGroupingByInitial(bookSkus,
									CultureInfo.CurrentUICulture,
									(BookSku bookSku) => { return bookSku.authorName; }, true);
				}
				return this.authors;
			}
		}

		public ObservableCollection<BookSku> BookSkus { get { return this.bookSkus; } }

		private bool CanAddMobyDick
		{
			get
			{
				return !isMobyDickInTheStore;
			}
		}

		private bool CanUpdateAtticus
		{
			get
			{
				return !hasAtticusBeenUpdated;
			}
		}

		public List<GroupByHelper<BookSku>> Genres
		{
			get
			{
				if (this.genres == null)
				{
					this.genres = GroupByHelper<BookSku>.CreateSortedGroupingByKeysPresent(bookSkus,
									(BookSku bookSku) => { return bookSku.genre; });
				}
				return this.genres;
			}
		}
		#endregion properties

		#region methods
		private void AddMobyDick()
		{
			bookSkus.Add(new BookSku("Moby-Dick", "Melville, Herman", "Adventure", "/Assets/CoverImages/two.png"));
			this.isMobyDickInTheStore = true;
			this.MobyDickCommand.RaiseCanExecuteChanged();
			this.authors = null;
			this.RaisePropertyChanged("Authors");
			this.genres = null;
			this.RaisePropertyChanged("Genres");
		}

		private void UpdateAtticus()
		{
			foreach (BookSku bookSku in this.bookSkus)
			{
				if (bookSku.Title == "Atticus")
				{
					bookSku.Title = "To Kill a Mockingbird";
					this.hasAtticusBeenUpdated = true;
					this.AtticusCommand.RaiseCanExecuteChanged();
					break;
				}
			}
		}
		#endregion methods

		#region constructors
		public BookstoreViewModel()
		{
			// Establish the invariant of owning a collection of BookSku.
			this.bookSkus = new ObservableCollection<BookSku>();

			if (System.ComponentModel.DesignerProperties.IsInDesignTool)
			{
				DataSource.LoadSampleData(ref this.bookSkus);
			}
			else
			{
				DataSource.LoadDataFromCloudService(ref this.bookSkus);
			}
		}
		#endregion constructors

		#region commands
		/// <summary>
		/// You can bind the Command property of a Button to a property of type RelayCommand. When the Button is clicked, the command will be
		/// invoked. The Button will be enabled as long as the command can execute.
		/// </summary>
		public RelayCommand AtticusCommand
		{
			get
			{
				if (this.atticusCommand == null)
				{
					// The RelayCommand is constructed with two parameters - the action to perform on invocation,
					// and the condition under which the command can execute. It's important to call RaiseCanExecuteChanged
					// on a command whenever its can-execute condition might have changed. Here, we do that in the TrialExperienceHelper_LicenseChanged
					// event handler.
					this.atticusCommand = new RelayCommand(
						param => this.UpdateAtticus(),
						param => this.CanUpdateAtticus);
				}
				return this.atticusCommand;
			}
		}

		public RelayCommand MobyDickCommand
		{
			get
			{
				if (this.mobyDickCommand == null)
				{
					// The RelayCommand is constructed with two parameters - the action to perform on invocation,
					// and the condition under which the command can execute. It's important to call RaiseCanExecuteChanged
					// on a command whenever its can-execute condition might have changed. Here, we do that in the TrialExperienceHelper_LicenseChanged
					// event handler.
					this.mobyDickCommand = new RelayCommand(
						param => this.AddMobyDick(),
						param => this.CanAddMobyDick);
				}
				return this.mobyDickCommand;
			}
		}
		#endregion commands
	}

	public class BookSku : PropertyChangedNotifier
	{
		#region fields
		internal string authorName;
		internal string genre;
		private string title;
		#endregion fields

		#region properties
		public string AuthorNameAndGenre
		{
			get
			{
				return this.authorName + " | " + this.genre;
			}
		}

		public ImageSource CoverImage
		{
			get
			{
				// this.CoverImagePath contains a path of the form "/Assets/CoverImages/one.png".
				return new BitmapImage(new Uri(this.CoverImagePath, UriKind.Relative));
			}
		}

		public string CoverImagePath { get; private set; }

		public string Title
		{
			get
			{
				return this.title;
			}
			internal set
			{
				this.SetPropertyAndRaisePropertyChanged(ref this.title, value);
			}
		}
		#endregion properties

		#region constructors
		public BookSku(string title, string authorName, string genre, string coverImagePath)
		{
			this.Title = title;
			this.authorName = authorName;
			this.genre = genre;
			this.CoverImagePath = coverImagePath;
		}
		#endregion constructors
	}

	internal static class DataSource
	{
		#region methods
		public static void LoadSampleData(ref ObservableCollection<BookSku> bookSkus)
		{
			bookSkus.Add(new BookSku("Emma", "Austen, Jane", "Romance", "/Assets/CoverImages/three.png"));
			bookSkus.Add(new BookSku("Mansfield Park", "Austen, Jane", "Romance", "/Assets/CoverImages/six.png"));
			bookSkus.Add(new BookSku("Persuasion", "Austen, Jane", "Romance", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("Pride and Prejudice", "Austen, Jane", "Romance", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Sense and Sensibility", "Austen, Jane", "Romance", "/Assets/CoverImages/seven.png"));
			bookSkus.Add(new BookSku("The Secret Garden", "Burnett, Frances Hodgson", "Children's", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("A Christmas Carol", "Dickens, Charles", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("Atticus", "Lee, Harper", "Drama", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("David Copperfield", "Dickens, Charles", "Drama", "/Assets/CoverImages/three.png"));
			bookSkus.Add(new BookSku("East of Eden", "Steinbeck, John", "Drama", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("Gone With The Wind", "Mitchell, Margaret", "Historical", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("Great Expectations", "Dickens, Charles", "Drama", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Little Dorrit", "Dickens, Charles", "Drama", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Oliver Twist", "Dickens, Charles", "Drama", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("Treasure Island", "Stevenson, R.L.", "Adventure", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("The Fellowship Of The Ring", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("The Hobbit", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("The Return Of The King", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("The Two Towers", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/six.png"));
		}

		public static void LoadDataFromCloudService(ref ObservableCollection<BookSku> bookSkus)
		{
			// In this simple app, we'll simulate real-world data access by loading sample data.
			DataSource.LoadSampleData(ref bookSkus);
		}
		#endregion methods
	}
}